Package: RcisTarget
Type: Package
Title: RcisTarget Identify transcription factor binding motifs enriched on a
    list of genes or genomic regions
Version: 1.22.0
Date: 2022-12-01
Author: Sara Aibar, Gert Hulselmans, Stein Aerts. Laboratory of Computational Biology. 
    VIB-KU Leuven Center for Brain & Disease Research. Leuven, Belgium
Maintainer: Sara Aibar <sara.aibar@kuleuven.vib.be>
Description: RcisTarget identifies transcription factor binding
    motifs (TFBS) over-represented on a gene list.
    In a first step, RcisTarget selects DNA motifs that are significantly 
    over-represented in the surroundings of the transcription start site (TSS) 
    of the genes in the gene-set. 
    This is achieved by using a database that contains genome-wide cross-species 
    rankings for each motif. The motifs that are then annotated to TFs and those
    that have a high Normalized Enrichment Score (NES) are retained. 
    Finally, for each motif and gene-set, 
    RcisTarget predicts the candidate target genes 
    (i.e. genes in the gene-set that are ranked above the leading edge).
URL: http://scenic.aertslab.org
Depends: R (>= 3.4)
Imports:
    AUCell (>= 1.1.6),
    BiocGenerics,
    data.table,
    graphics,
    GenomeInfoDb,
    GenomicRanges,
    arrow (>= 2.0.0),
    dplyr,
    tibble,
    GSEABase,
    methods,
    R.utils,
    stats,
    SummarizedExperiment,
    S4Vectors,
    utils
Enhances: doMC, doRNG, zoo
Suggests:
    Biobase,
    BiocStyle,
    BiocParallel,
    doParallel,
    DT,
    foreach,
    gplots,
    rtracklayer,
    igraph,
    knitr,
    RcisTarget.hg19.motifDBs.cisbpOnly.500bp, 
    rmarkdown,
    testthat,
    visNetwork
License: GPL-3
BugReports: https://github.com/aertslab/RcisTarget/issues
biocViews: GeneRegulation, MotifAnnotation, Transcriptomics, Transcription,
    GeneSetEnrichment, GeneTarget
LazyData: FALSE
VignetteBuilder: knitr
RoxygenNote: 7.2.1
