
# RcisTarget
RcisTarget is an R-package to identify transcription factor (TF) binding motifs over-represented on a gene list. 

*Availability*: 

- The newest stable version of RcisTarget is available in *[Bioconductor](https://bioconductor.org/packages/RcisTarget)*. 
The package also contains a **tutorial** [(vignette)](https://bioconductor.org/packages/release/bioc/vignettes/RcisTarget/inst/doc/RcisTarget.html) with information on how to run RcisTarget and how to interprete its results.

- This Github repository is meant mostly for development. "Use at your own risk" :-). Developers are welcome to contribute. 

----


*Databases*: 

The databases required to run (R)cisTarget are available at https://resources.aertslab.org/cistarget/ (see the vignette for details). Alternative *mirror*: https://resources-mirror.aertslab.org/cistarget/.

A tutorial on how to **create databases** with different settings (e.g. **new species**) is available at https://github.com/aertslab/create_cisTarget_databases
